package at.ac.tuwien.cs.ztimeview.core;

import java.util.*;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.DefaultMutableTreeNode;

import at.ac.tuwien.cs.ztimeview.time.TimeInterval;
import at.ac.tuwien.cs.ztimeview.time.Interval;

/**
Class representing a Task
 
@author Wolfgang Aigner
@version 0.3

 created: 2004-05-19
 last change: 2004-06-04
*/
public class Task extends DefaultMutableTreeNode implements Interval {
	
	static int INDENT_SIZE = 2; // indent size for printing to screen
	
	String name = null; // task name
	Date start = null; //start time
	Date end = null; // end tme
			
	//////////////////// CONSTRUCTORS ////////////////////
	
	/** Default constructor */
	public Task() throws IllegalArgumentException {
		this("", null, null);
	}
	
	/** Constructor
	@param name task name
	@param interval time interval */
	public Task(String name, Interval interval) throws IllegalArgumentException {
		this(name, interval.getStart(), interval.getEnd());
	}
	
	/** Constructor
	@param name task name
	@param start start time of interval
	@param end end time of interval */
	public Task(String name, Date start, Date end) throws IllegalArgumentException {
		super();
		this.setName(name);
		this.setStart(start);
		this.setEnd(end);
	}
	
	//////////////////// METHODS ////////////////////
	
	/** Print task hierarchy textually to the screen */
	public void printHierarchy() {
		this.printHierarchy(0);
	}
	
	/** Print task hierarchy textually to the screen
	@param indent number of spaces for indentation */
	protected void printHierarchy(int indent) {
		// --- make indent ---
		for(int i=0; i<indent; i++) System.out.print(" ");
		
		// --- print task name, start and end ---
		System.out.print(this.toString() + "\n");
		
		// --- print subplan steps ---
		Task childTask = null;
		
		for(Enumeration child = this.children(); child.hasMoreElements();) {
			childTask = ((Task)(child.nextElement()));
			childTask.printHierarchy(indent + INDENT_SIZE);
		}
	}
	
	public String toString() {
		return (this.getName() + " (" + this.getStart() + " - " + this.getEnd() + ")");
	}
	
	//////////////////// GET/SET METHODS ////////////////////
	
	/** Set task name
	@param name task name */
	public void setName(String name) {
		this.name = name;
	}
	
	/** Get task name
	@return task name */
	public String getName() {
		return name;
	}
	
	//////////////////// Interval METHODS ////////////////////
	
	/** Set the start of the interval 
	@param start beginning of interval */
	public void setStart(Date start) {
		this.start = start;
	}
	
	/** Get the start of the interval 
	@return beginning of interval */
	public Date getStart() {
		return this.start;
	}

	/** Set the end of the interval 
	@param end end of interval */
	public void setEnd(Date end) {
		this.end = end;
	}
	
	/** Get the end of the interval 
	@return end of interval */
	public Date getEnd() {
		return this.end;
	}

}
