package at.ac.tuwien.cs.ztimeview.time;

import javax.swing.Timer;
import java.util.Date;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Color;

/**
Class representing a Clock
  
@author Wolfgang Aigner
@version 0.2
 
 created: 2003-03-26
 last change: 2003-04-03
*/
public class Clock extends Timer implements ActionListener {
	/** Default notification interval */
	static final int DEFAULT_NOTIFICATION_INTERVAL = 1000; // 1 s
	
	/** actual time */
	Date time = null;
	
	/** default color */
	public static final Color DEFAULT_COLOR = Color.red;
	
	//////////////////// CONSTRUCTORS ////////////////////
	
	/** Constructor */
	public Clock() {
		this(new Date(), DEFAULT_NOTIFICATION_INTERVAL);
	}
	
	/** Constructor
	@param time actual time */
	public Clock(Date time) {
		this(time, DEFAULT_NOTIFICATION_INTERVAL);
	}
	
	/** Constructor
	@param time actual time 
	@param delay time between notifications (ms) */
	public Clock(Date time, int delay) {
		super(delay, null);
		this.time = time;
		this.addActionListener(this);
	}
	
	//////////////////// LISTENER METHODS ////////////////////
	
	/** Update time
	@param e event object */
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == this) {
			time = new Date(time.getTime() + this.getDelay());
		}
	}
	
	//////////////////// GET/SET METHODS ////////////////////
	
	/** Set the clock's time
	@param time actual time */
	public void setTime(Date time) {
		this.time = time;
	}
	
	/** Get the clock's time
	@return clock's actual time */
	public Date getTime() {
		return this.time;
	}
	
}
