package at.ac.tuwien.cs.ztimeview.ui;

import java.util.Iterator;
import java.io.File;
import java.awt.Color;
import java.awt.event.InputEvent;
import java.awt.geom.Point2D;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolox.PFrame;
import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PZoomEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.activities.PActivity;
import edu.umd.cs.piccolo.event.PInputEventFilter;
import edu.umd.cs.piccolox.swing.PScrollPane;

import at.ac.tuwien.cs.ztimeview.core.Task;
import at.ac.tuwien.cs.ztimeview.test.TaskGenerator;
import at.ac.tuwien.cs.ztimeview.util.PLifeLinePlusFactory;
import at.ac.tuwien.cs.ztimeview.util.PLifeLineFactory;

/**
LifeLine+ example application
shows a LifeLine+ hierarchy
including Overview+Detail display
  
@author Wolfgang Aigner, Martin Tomitsch
@version 0.3
 
 created: 2004-06-02
 last change: 2004-06-08
*/
public class PLifeLinePlusExample extends PFrame {

	// command line argument for lifeline creation
	final static String LIFE_LINE = "lifeline";
	
	// command line argument for lifeline+ creation
	final static String LIFE_LINE_PLUS = "lifelineplus";
	
	// command line argument for planningline creation
	final static String PLANNING_LINE = "planningline";
	
	// animation speed
	final static int ANIMATION_SPEED = 1000;
	
	// detail display color
	final static Color DETAIL_RECT_COLOR = Color.red;
	
	// start point for task hierarchy
	Task topTask;
	
	// kind of example (lifeline, lifelineplus, or planningline)
	String kind = null;
	
	/** Default constructor */
	public PLifeLinePlusExample() {
		super();	
	}
	
	public PLifeLinePlusExample(Task topTask, String kind) {
		this.topTask = topTask;
		this.kind = kind;
	}
	
	/** PFrame initialization code */
	public void initialize() {
		// --- set window size ---
		this.setSize(800, 600);
		this.setLocation(0, 0);
		this.validate();
		
		// --- add scroll bars ---
		final PScrollPane scrollPane = new PScrollPane(getCanvas());
		getContentPane().add(scrollPane);
		
		PTimeLayer lifeLine = null;
		if(this.kind.equalsIgnoreCase(LIFE_LINE)) {
			// --- generate LifeLine ---
			lifeLine = PLifeLineFactory.getPLifeLineFor(topTask);
		} else {
			// --- generate LifeLine+ / PlanningLine ---
			lifeLine = PLifeLinePlusFactory.getPLifeLinePlusFor(topTask);
			((PLifeLinePlus) lifeLine).expand();
		}
			
		// Add the node to the canvas layer so that it
		// will be displayed on the screen.
		getCanvas().getLayer().addChild(lifeLine);
		
		final PCamera camera = getCanvas().getCamera();
		
		// --- add tooltip node ---
		final PText tooltipNode = new PText();
		
		tooltipNode.setPickable(false);
		camera.addChild(tooltipNode);
		
		camera.addInputEventListener(new PBasicInputEventHandler() {	
			public void mouseMoved(PInputEvent event) {
				updateToolTip(event);
			}

			public void mouseDragged(PInputEvent event) {
				updateToolTip(event);
			}

			public void updateToolTip(PInputEvent event) {
				PNode n = event.getInputManager().getMouseOver().getPickedNode();
				String tooltipString = (String) n.getClientProperty("tooltip");
				Point2D p = event.getCanvasPosition();
				
				event.getPath().canvasToLocal(p, camera);
				
				tooltipNode.setText(tooltipString);
				tooltipNode.setOffset(p.getX() + 8, 
					p.getY() - 8);
			}
		});
		
		// --- zoom & pan on selection ---
		// selection handler
		PBasicInputEventHandler selectionEventHandler = new PBasicInputEventHandler() {
		    public void mousePressed(PInputEvent event) {
				if (!event.isShiftDown() && event.getPickedNode() != null) {
				    PBounds b = event.getPickedNode().getGlobalFullBounds();
				    growBounds(b, 0.1d);
				    PActivity a = event.getCamera().animateViewToCenterBounds(b, true, ANIMATION_SPEED);
				    event.getCamera().addActivity(a);
				}
		    };
		};
		//selectionEventHandler.setEventFilter(new PInputEventFilter(InputEvent.BUTTON1_MASK));
		selectionEventHandler.setEventFilter(new PInputEventFilter(InputEvent.BUTTON1_MASK, InputEvent.ALT_MASK));
		lifeLine.addInputEventListener(selectionEventHandler);
      
		// -- add scale ---
		PScaleLayer scaleLayer = new PScaleLayer((int)this.getCanvas().getWidth(),lifeLine.getStart(),lifeLine.getEnd());
		getCanvas().getCamera().addChild(scaleLayer);
		
		// -- add ruler ---
		PRulerLayer rulerLayer = new PRulerLayer((int)this.getCanvas().getWidth(),(int)this.getCanvas().getHeight(),lifeLine.getStart(),lifeLine.getEnd());
		getCanvas().getCamera().addChild(rulerLayer);
		
		getCanvas().getPanEventHandler().setEventFilter(new PInputEventFilter(InputEvent.BUTTON1_MASK & InputEvent.SHIFT_MASK, InputEvent.ALT_MASK));
			
		// --- overview window ---	
		final PCamera overviewCamera = new PCamera();
		overviewCamera.addLayer(lifeLine);
		getCanvas().getRoot().addChild(overviewCamera); 
		
		PLayer blockLayer = new PLayer(); // block input to overview layer
		blockLayer.setBounds(lifeLine.getBounds());
		PBasicInputEventHandler blockEventHandler = new PBasicInputEventHandler() {
		    public void mousePressed(PInputEvent event) {
				/* do nothing */
		    };
		};
		blockEventHandler.setEventFilter(new PInputEventFilter(InputEvent.BUTTON1_MASK));
		blockLayer.addInputEventListener(blockEventHandler);
		overviewCamera.addLayer(blockLayer);
		
		PCanvas overview = new PCanvas();
		overview.setCamera(overviewCamera);
		PFrame overviewWindow = new PFrame(false, overview);
		overviewWindow.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		overviewWindow.setSize(200, 200);
		overviewWindow.setLocation(800, 0);
		
		final PPath detail = PPath.createRectangle(1,1,100,100);
		detail.setPaint(null);
		detail.setStrokePaint(DETAIL_RECT_COLOR);
		overviewCamera.addChild(detail);
		
		// update detail display in overview window if view transform changes
		camera.addPropertyChangeListener(PCamera.PROPERTY_VIEW_TRANSFORM, new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				detail.setBounds(overviewCamera.viewToLocal(camera.getViewBounds()));
			}
		});
       
		// --- display windows ---
		getContentPane().validate();
		overviewWindow.validate();
		
		// --- zoom overview window to extent ---
		PBounds bounds = lifeLine.getFullBounds();
		growBounds(bounds,0.1d);
		overviewCamera.animateViewToCenterBounds(bounds, true, 0);
			
		// --- zoom to extent ---
		doViewAll(getCanvas());
	}
	
	/** Scale view to show everything 
	@param c PCanvas to zoom */
	public void doViewAll(PCanvas c) {
		PLayer l = c.getLayer();
		PCamera cam = c.getCamera();
		
		PBounds bounds = l.getChild(0).getBounds();
		growBounds(bounds,0.1d);
		cam.animateViewToCenterBounds(bounds, true, ANIMATION_SPEED);
	}
    
	/** Grow the given bounds by percentage
	@param b PBounds to enlarge / shrink
	@param percentage enlargement / shring factor */
	static void growBounds(PBounds b, double percentage) {
		b.x -= b.width*percentage;
		b.y -= b.height*percentage;
		b.width += b.width*percentage*2;
		b.height += b.height*percentage*2;
	}
	
	/** Main method to be called from command line 
	@param args command line arguments */
	public static void main(String[] args) {
		String usage = "PLifeLinePlusExample (lifeline | lifelineplus | planningline) topDir";
		
		if(args.length < 2) { // wrong number of command line arguments
			System.out.println(usage);
		} else {
			// --- create test tasks ---
			Task task = null;
			try {
				task = TaskGenerator.generateTaskFromDir(new File(args[1]), 
					args[0].equalsIgnoreCase(PLANNING_LINE));
			} catch(java.lang.NullPointerException e) {
				System.err.println(e.getMessage());
			}
			// --- instantiate example ---
			PLifeLinePlusExample pe = new PLifeLinePlusExample(task, args[0]);
		}
	}
}