package at.ac.tuwien.cs.ztimeview.ui;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.Font;
import java.util.Date;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import java.util.Vector;
import java.util.Iterator;
import java.text.SimpleDateFormat;
import javax.swing.JLabel;
import java.awt.event.InputEvent;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolo.util.PPaintContext;
import edu.umd.cs.piccolo.util.PDimension;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.event.PInputEventFilter;
import edu.umd.cs.piccolo.PCanvas;

/**
implementing the ruler

@author Martin Tomitsch
@version 0.1
  
 created: 2004-06-07
 last change: 2004-06-07
*/
public class PRulerLayer extends PLayer {
	
	private int BACKGROUNDHEIGHT=40;
	private int AXISTOP = 40; //top y position of axis
	Date scaleStart = new Date();
	Date scaleEnd = new Date();
	long currentStart = 0;
	long currentEnd = 0;
	int canvasWidth; //width of canvas in pixel
	int canvasHeight; //height of canvas in pixel
	
	PCanvas canvas=null;
	
	int rulerPos=0; //x position of ruler
	PPath ruler = new PPath(); //ruler node
	
	PText label = new PText(""); //label for date
		
	// precision of display width[px] = duration[ms]*PRECISION
	private double PRECISION = 0.000001; 
	
	/** constructor */
	public PRulerLayer(int canvasWidth, int canvasHeight, Date start, Date end) {
		super();
		this.scaleStart = start;
		this.currentStart = start.getTime();
		this.scaleEnd = end;
		this.currentEnd = start.getTime();
		this.canvasWidth = canvasWidth;
		this.canvasHeight = canvasHeight;
		this.canvas = canvas;
		
		rulerPos = (int)this.canvasWidth/2;
		
		//paint background box
		PPath background;
		background = PPath.createRectangle(0, canvasHeight-BACKGROUNDHEIGHT, this.canvasWidth, this.canvasHeight);
		background.setPaint(Color.WHITE);
		background.setStroke(null);
		this.addChild(0,background);
		
		//paint ruler
		ruler.moveTo(this.canvasWidth/2,AXISTOP);
		ruler.lineTo(this.canvasWidth/2,this.canvasHeight-BACKGROUNDHEIGHT);
		//ruler.setPathToRectangle(this.canvasWidth/2-10,this.canvasHeight-BACKGROUNDHEIGHT,20,10);
		ruler.setStrokePaint(Color.red);
		
		this.addChild(ruler);

		//define event handlers		
		PBasicInputEventHandler rulerEventHandler = new PBasicInputEventHandler() {
			public void mouseDragged(PInputEvent aEvent) {
				super.mouseDragged(aEvent);
				if (aEvent.isAltDown()) {
					rulerPos = (int)aEvent.getCanvasPosition().getX();
					drawRuler();
					writeLabel();
				}
			}
		};
		
		rulerEventHandler.setEventFilter(new PInputEventFilter(InputEvent.BUTTON1_MASK & InputEvent.ALT_MASK));
		ruler.addInputEventListener(rulerEventHandler);
		
		this.addChild(label);
		writeLabel();
	}

	/** draw ruler */
	private void drawRuler() {
		//System.out.print("drag\n"); //debug
		//repaint ruler
		ruler.reset();
		ruler.moveTo((float)rulerPos,AXISTOP);
		ruler.lineTo((float)rulerPos,canvasHeight-BACKGROUNDHEIGHT);		
		ruler.updateBoundsFromPath();
		writeLabel();	
	}
	
	/** write label for date */
	private void writeLabel() {
		GregorianCalendar actTime = new GregorianCalendar();
		actTime.setTimeZone(TimeZone.getTimeZone("GMT"));
		actTime.setTimeInMillis((long)(currentStart+(currentEnd-currentStart)/canvasWidth*rulerPos));
		SimpleDateFormat formatter = new SimpleDateFormat("dd.MM.yyyy HH:mm");
		label.setText(formatter.format(actTime.getTime()));
		
		// helper component to determine labelwidth
		JLabel lbl_caption = new JLabel("");
		lbl_caption.setFont(label.getFont());
		lbl_caption.setText(label.getText()); 
		int currentLabelSize = (int)lbl_caption.getPreferredSize().getWidth(); //get label width

		label.setBounds(rulerPos-currentLabelSize/2,canvasHeight-35,label.getBounds().getSize().getWidth(),label.getBounds().getSize().getHeight());
	}
	
	//****************************************************************
	// Structure - Methods for manipulating and traversing the 
	// parent child relationship
	// 
	// Most of these methods won't need to be overridden by subclasses
	// but you will use them frequently to build up your node structures.
	//****************************************************************
	 
	/**
	 * Add a node to be a new child of this node at the specified index.
	 * If child was previously a child of another node, it is removed 
	 * from that node first.
	 * 
	 * @param child the new child to add to this node
	 */
	public void addChild(int index, PNode child) {
		super.addChild(index, child);
	}
	
	public void fullPaint(PPaintContext aPaintContext) {
		double s = aPaintContext.getScale();
		PBounds vb = aPaintContext.getCamera().getViewBounds();
		
		canvasWidth = (int)aPaintContext.getLocalClip().getWidth(); //get size
		
		long x = (long) (vb.getOrigin().getX() / PRECISION); 
		long width = (long) (vb.getSize().getWidth() / PRECISION);
		
		Date scaleCurrentStart = new Date(scaleStart.getTime() + x + (long)width);
		Date scaleCurrentEnd = new Date(scaleStart.getTime() + x + width - (long)width);
		
		// delete all children first
		//this.removeAllChildren();
		
		currentEnd = scaleCurrentStart.getTime();
		currentStart = scaleCurrentEnd.getTime();
		
		drawRuler();
				
		super.fullPaint(aPaintContext);

	}
	
	
	//////////////////// GET/SET METHODS ////////////////////
	
}
