package at.ac.tuwien.cs.ztimeview.core;

import java.util.Date;
import at.ac.tuwien.cs.ztimeview.time.TimeAnnotation;
import at.ac.tuwien.cs.ztimeview.time.ActivePlanningInterval;
import at.ac.tuwien.cs.ztimeview.time.ReferencePoint;

/**
Class representing a Task including temporal uncertainties
 
@author Wolfgang Aigner
@version 0.1
 
created: 2004-06-06
last change: 2004-06-06
*/
public class PlanningTask extends Task implements ActivePlanningInterval {
	
	TimeAnnotation timeAnnotation = null; // timing attributes
	
	public PlanningTask() {
		super();
		this.timeAnnotation = new TimeAnnotation();
		this.timeAnnotation.setRef(new ReferencePoint());
	}
	
	//////////////////// GET/SET METHODS ////////////////////
	
	/** Set time annotation
	@param timeAnnotation timing attributes of plan */
	public void setTimeAnnotation(TimeAnnotation timeAnnotation) {
		this.timeAnnotation = timeAnnotation;
	}
	
	/** Get time annotation
	@return timing attributes of plan */
	public TimeAnnotation getTimeAnnotation() {
		return (TimeAnnotation) this.timeAnnotation;
	}
	
	public String toString() {
		return (this.getName() + " (EST: " + this.getEarliestStartingTime() 
			+ " LST: " + this.getLatestStartingTime() 
			+ " EFT: " + this.getEarliestFinishingTime() 
			+ " LFT: " + this.getLatestFinishingTime() 
			+ ")");
	}
	
	//////////////////// PlanningInterval METHODS ////////////////////
	
	/** Set earliest starting time
	@param est earliest starting time */
	public void setEarliestStartingTime(Date est) {
		this.timeAnnotation.setEarliestStartingTime(est);
	}
	
	/** Get earliest starting time
	@return earliest starting time */
	public Date getEarliestStartingTime() {
		return this.timeAnnotation.getEarliestStartingTime();
	}
	
	/** Set latest starting time
	@param lst latest starting time */
	public void setLatestStartingTime(Date lst) {
		this.timeAnnotation.setLatestStartingTime(lst);
	}
	
	/** Get latest starting time
	@return latest starting time */
	public Date getLatestStartingTime() {
		return this.timeAnnotation.getLatestStartingTime();
	}
	
	/** Set earliest finishing time
	@param eft earliest finishing time */
	public void setEarliestFinishingTime(Date eft) {
		this.timeAnnotation.setEarliestFinishingTime(eft);
	}
	
	/** Get earliest finishing time
	@return earliest finishing time */
	public Date getEarliestFinishingTime() {
		return this.timeAnnotation.getEarliestFinishingTime();
	}	
	
	/** Set latest finishing time
	@param lft latest finishing time */
	public void setLatestFinishingTime(Date lft) {
		this.timeAnnotation.setLatestFinishingTime(lft);
	}
	
	/** Get latest finishing time
	@return latest finishing time */
	public Date getLatestFinishingTime() {
		return this.timeAnnotation.getLatestFinishingTime();
	}
	
	/** Set minimum duration
	@param minDu minimum duration in milliseconds */
	public void setMinimumDuration(Long minDu) {
		this.timeAnnotation.setMinimumDuration(minDu);
	}
	
	/** Get minimum duration
	@return minimum duration in milliseconds */
	public Long getMinimumDuration() {
		return this.timeAnnotation.getMinimumDuration();
	}
	
	/** Set maximum duration
	@param maxDu maximum duration in milliseconds */
	public void setMaximumDuration(Long maxDu) {
		this.timeAnnotation.setMaximumDuration(maxDu);
	}
	
	/** Get maximum duration
	@return maximum duration in milliseconds */
	public Long getMaximumDuration() {
		return this.timeAnnotation.getMaximumDuration();
	}
	
	/** Get earliest instant of time annotation */
	public Date getEarliestBegin() {
		return this.timeAnnotation.getEarliestBegin();
	}
	
	/** Get latest instant of time annotation */
	public Date getLatestEnd() {
		return this.timeAnnotation.getLatestEnd();
	}	
	
	/** Get beginning of maxDu interval */
	public Date getMaxDuBegin() {
		return this.timeAnnotation.getMaxDuBegin();
	}
	
	/** Get beginning of minDu interval */
	public Date getMinDuBegin() {
		return this.timeAnnotation.getMinDuBegin();
	}	
	
	/** Get end of maxDu interval */
	public Date getMaxDuEnd(){
		return this.timeAnnotation.getMaxDuEnd();
	}
	
	/** Get end of minDu interval */
	public Date getMinDuEnd(){
		return this.timeAnnotation.getMinDuEnd();
	}		

}
