package at.ac.tuwien.cs.ztimeview.time;

import java.util.Date;

/**
Interface for objects representing a planning interval
  
@author Wolfgang Aigner
@version 0.3
  
 created: 2003-03-18
 last change: 2003-03-27
*/
public interface PlanningInterval extends Timed {
	/** Set earliest starting time
	@param est earliest starting time */
	public void setEarliestStartingTime(Date est);
	
	/** Get earliest starting time
	@return earliest starting time */
	public Date getEarliestStartingTime();
	
	/** Set latest starting time
	@param lst latest starting time */
	public void setLatestStartingTime(Date lst);
	
	/** Get latest starting time
	@return latest starting time */
	public Date getLatestStartingTime();
	
	/** Set earliest finishing time
	@param eft earliest finishing time */
	public void setEarliestFinishingTime(Date eft);
	
	/** Get earliest finishing time
	@return earliest finishing time */
	public Date getEarliestFinishingTime();
	
	/** Set latest finishing time
	@param lft latest finishing time */
	public void setLatestFinishingTime(Date lft);
	
	/** Get latest finishing time
	@return latest finishing time */
	public Date getLatestFinishingTime();
	
	/** Set minimum duration
	@param minDu minimum duration in milliseconds */
	public void setMinimumDuration(Long minDu);
	
	/** Get minimum duration
	@return minimum duration in milliseconds */
	public Long getMinimumDuration();
	
	/** Set maximum duration
	@param maxDu maximum duration in milliseconds */
	public void setMaximumDuration(Long maxDu);
	
	/** Get maximum duration
	@return maximum duration in milliseconds */
	public Long getMaximumDuration();
	
	/** Get earliest instant of time annotation */
	public Date getEarliestBegin();
	
	/** Get latest instant of time annotation */
	public Date getLatestEnd();	
	
	/** Get beginning of maxDu interval */
	public Date getMaxDuBegin();
	
	/** Get end of maxDu interval */
	public Date getMaxDuEnd();
	
	/** Get beginning of minDu interval */
	public Date getMinDuBegin();
	
	/** Get end of minDu interval */
	public Date getMinDuEnd();		
}
