package at.ac.tuwien.cs.ztimeview.ui;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.Font;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Vector;
import java.util.Iterator;
import java.text.SimpleDateFormat;
import javax.swing.JLabel;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolo.util.PPaintContext;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.nodes.PPath;

/**
Piccolo Layer containing the scale
  
@author Martin Tomitsch
@version 0.2
  
 created: 2004-06-03
 last change: 2004-06-06
*/
public class PScaleLayer extends PLayer {
	
	private int BACKGROUNDHEIGHT=70;
	private int LABELSIZE = 50; //length of label text
	private int AXISTOP = 40; //top y position of axis
	Date scaleStart = new Date();
	Date scaleEnd = new Date();
	int canvasWidth; //width of canvas in pixel
		
	// precision of display width[px] = duration[ms]*PRECISION
	private double PRECISION = 0.000001; 
	
	public PScaleLayer(int canvasWidth, Date start, Date end) {
		super();
		scaleStart = start;
		scaleEnd = end;
		this.canvasWidth = canvasWidth;
	}
	
	/** paint axis: creating children, calculating tick intervals, drawing axis and labels
		@param start lower date
		@param end upper date
		@param bounds */
	public void drawAxis (long start, long end, long length, long x, double space) {
		PPath background;
		background = PPath.createRectangle(0, 0, canvasWidth, BACKGROUNDHEIGHT);
		background.setPaint(Color.WHITE);
		background.setStroke(null);
		this.addChild(0,background);
		
		//print start and end date
		SimpleDateFormat formatter = new SimpleDateFormat("dd.MM.yyyy HH:mm");
		PText startLabel = new PText(formatter.format(new Date(start)));
		startLabel.setFont(new Font(null, Font.BOLD, 12)); //change font to bold for main labels
		this.addChild(startLabel);
		PText endLabel = new PText(formatter.format(new Date(end)));
		endLabel.setBounds(canvasWidth-100,0,endLabel.getBounds().getSize().getWidth(),endLabel.getBounds().getSize().getHeight());
		endLabel.setFont(new Font(null, Font.BOLD, 12)); //change font to bold for main labels
		this.addChild(endLabel);
		
		PPath axisLine = new PPath();
		//axisLine.setStrokePaint(Color.darkGray);
		axisLine.moveTo((float)(canvasWidth*space),AXISTOP);
		axisLine.lineTo((float)(canvasWidth-canvasWidth*space),AXISTOP);
		
		PTimeScale timescale = new PTimeScale(start, end, (int)(canvasWidth-2*canvasWidth*space), (int)(canvasWidth*space));
		Vector ticks = new Vector(timescale.calculateTicks()); //list of ticks
		PTick tick = new PTick();
		PText label;
		
		// helper component to determine labelwidth
		JLabel lbl_caption = new JLabel("");
		int currentLabelSize = 0;
		
		Iterator each = ticks.iterator();
		while (each.hasNext()) {
			tick = (PTick)each.next();
			
			//write label text
			if (tick.getLabel().compareTo("") != 0) {
				label = new PText(tick.getLabel());
				lbl_caption.setFont(label.getFont());
				lbl_caption.setText(tick.getLabel()); //get label width
				currentLabelSize = (int)lbl_caption.getPreferredSize().getWidth();
				//label.setStrokePaint(Color.darkGray);
				label.setBounds(tick.getCanvasX()-currentLabelSize/2,AXISTOP-17,label.getBounds().getSize().getWidth(),label.getBounds().getSize().getHeight());
				//draw tick for the label
				axisLine.moveTo((float)tick.getCanvasX(),AXISTOP);
				axisLine.lineTo((float)tick.getCanvasX(),AXISTOP+15);
				if (tick.getMainLabel())
					label.setFont(new Font(null, Font.BOLD, 12)); //change font to bold for main labels
				this.addChild(label);
			}
			
			else {
				//draw tick
				axisLine.moveTo((float)tick.getCanvasX(),AXISTOP);
				if (tick.getSmallerGranularity()) //tick for smaller granularity
					axisLine.lineTo((float)tick.getCanvasX(),AXISTOP+7);
				else
					axisLine.lineTo((float)tick.getCanvasX(),AXISTOP+15);
			}
		}

		this.addChild(axisLine);

	}
	
	//****************************************************************
	// Structure - Methods for manipulating and traversing the 
	// parent child relationship
	// 
	// Most of these methods won't need to be overridden by subclasses
	// but you will use them frequently to build up your node structures.
	//****************************************************************
	 
	/**
	 * Add a node to be a new child of this node at the specified index.
	 * If child was previously a child of another node, it is removed 
	 * from that node first.
	 * 
	 * @param child the new child to add to this node
	 */
	public void addChild(int index, PNode child) {
		super.addChild(index, child);
	}
	
	public void fullPaint(PPaintContext aPaintContext) {
		double s = aPaintContext.getScale();
		PBounds vb = aPaintContext.getCamera().getViewBounds();
		
		canvasWidth = (int)aPaintContext.getLocalClip().getWidth(); //get size
		
		long x = (long) (vb.getOrigin().getX() / PRECISION); 
		long width = (long) (vb.getSize().getWidth() / PRECISION);
		double space = 0; // space to the left and right of the axis (in percent)
		
		Date scaleCurrentStart = new Date(scaleStart.getTime() + x + (long)(width*space));
		Date scaleCurrentEnd = new Date(scaleStart.getTime() + x + width - (long)(width*space));
		
		// delete all children first
		this.removeAllChildren();
		
		drawAxis(scaleCurrentStart.getTime(), scaleCurrentEnd.getTime(), width, x, space);
		
		//GregorianCalendar c = new GregorianCalendar();

		/*c.setTime(scaleCurrentEnd);
		label2.setBounds(aPaintContext.getLocalClip().getWidth() - label2.getWidth(), this.getY(), this.getWidth(), this.getHeight());	
		label2.setText(Integer.toString(c.get(c.DAY_OF_MONTH))
			+ '.' 
			+ Integer.toString(c.get(c.MONTH) + 1) // return value is pointer into field, starting with 0
			+ '.' 
			+ Integer.toString(c.get(c.YEAR)));*/
		
		super.fullPaint(aPaintContext);
	}
	
	
	//////////////////// GET/SET METHODS ////////////////////
}
