package at.ac.tuwien.cs.ztimeview.util;

import java.util.Hashtable;
import java.awt.Color;

/**
Color manager that holds color values for entered objects
  
@author Wolfgang Aigner
@version 0.4
  
 created: 2003-03-04
 last change: 2004-05-21
*/
public class ColorManager {
	static IColorPalette palette = null;
	static Hashtable map = null;
	
	final static String DEFAULT_PALETTE_NAME = "at.ac.tuwien.cs.ztimeview.util.MyColorPalette";

	//////////////////// STATIC INITIALISER ////////////////////
	
	static {
		map = new Hashtable();
		try {
			palette = (IColorPalette) 
				Class.forName(DEFAULT_PALETTE_NAME).newInstance();
		} catch(Exception e) {
			System.err.println("Palette class not found. " + e.getMessage());
		}
	}
	
	//////////////////// METHODS ////////////////////
	
	/** Set color palette 
	@param cPalette color palette */
	public static void setPalette(IColorPalette cPalette) {
		palette = cPalette;
	}
	
	/** Get the color for the given object
	@param obj object to get color for
	@return object's color */
	public static Color getColorFor(Object obj) {
		Color color = null;
		
		if(obj != null) {
			Object c = map.get(obj);
			if(c == null) { // no entry for given object - create a new one
				color = palette.getNextColor();
				map.put(obj, color);
			}
			else {
				color = (Color) c;
			}
		}
		
		return color;
	}
	
	/** Transform the given Color to grayscale
	@param c input color
	@return grayscaled input color */
	public static Color getGrayColor(Color c) {
		float[] hsbColor = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
		float brightness = hsbColor[2];
		if(brightness > 0.9f) { // avoid white
			brightness = 0.8f;
		}
		return Color.getHSBColor(hsbColor[0], 0, brightness);
	}
	
	/** Brighten up the given color
	@param c input color
	@return brightened input color */
	public static Color brighter(Color c) {
		float[] hsbColor = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
		return Color.getHSBColor(hsbColor[0], hsbColor[1], (float) 0.95);
	}
	
	/** Darken the given color
	@param c input color
	@return darkened input color */
	public static Color darker(Color c) {
		float[] hsbColor = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
		return Color.getHSBColor(hsbColor[0], hsbColor[1], (float) 1.05);
	}
	
	/** Get the default foreground color
	@return default foreground color */
	public static Color getDefaultForeground() {
		Color c = Color.black;
		float[] hsbColor = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
		return Color.getHSBColor(hsbColor[0], hsbColor[1], (float) 0.1);
	}
	
	/** Get the foreground color in case of selection
	@return selection foreground color */
	public static Color getSelectedForeground() {
		Color c = Color.white;
		float[] hsbColor = Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), null);
		return Color.getHSBColor(hsbColor[0], hsbColor[1], (float) 0.9);
	}
	
}
