/**************************************************
VU XML, XSL and Web Applications

Lab 1

Name: Wolfgang Aigner
Matr.Nr.: 9755342

Created at: 29.10.2001
Last change: 30.10.2001
**************************************************/

package SimpleParser;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.*;
import javax.xml.parsers.*;

/** Simple XML parser using JAXP and SAX.  
Prints out elements with their content and attributes.
Warnings and errors are just printed to the screen. */
public class SimpleJAXPSAXParser extends DefaultHandler { 
    private SAXParserFactory spf = null; 
    private SAXParser saxParser = null; 

    // Constructor
    public SimpleJAXPSAXParser(boolean validation) throws Exception { 
	// Create a JAXP SAXParserFactory and configure it 
	spf = SAXParserFactory.newInstance(); 
	spf.setValidating(validation); 
	saxParser = spf.newSAXParser(); 
    } 

    // Main method - Creates a parser and parses the xml file given by the command line
    public static void main (String[] args) {
	try {
	    SimpleJAXPSAXParser p = new SimpleJAXPSAXParser(false);
	    p.parseDocument(args[0]);
        }catch(Exception e) {
	    System.err.println("SimpleJAXPSAXParser - program error: " + e);
	}
    }
	
    // parse the document 
    public void parseDocument( String filename ) throws Exception { 
	// Get the encapsulated SAX XMLReader from the parser 
	XMLReader xmlReader = saxParser.getXMLReader(); 
    
	// Set the ContentHandler of the XMLReader 
	xmlReader.setContentHandler(this); 
    
	// Set an ErrorHandler before parsing 
	xmlReader.setErrorHandler(this); 
    
	// Tell the XMLReader to parse the XML document
	xmlReader.parse(filename); 
    }  
    
    // begin of document
    public void startDocument() {
	System.out.println("--- Starting to parse document ---");
    }

    // begin of element
    public void startElement(String namespaceURI, String localName, String qName, Attributes attrs) { 
	System.out.print("<" + qName);

	// print attributes
	for(int i=0; i<attrs.getLength(); i++) {
	    System.out.print(" " + attrs.getQName(i) + "=\"" + attrs.getValue(i) + "\"");
	}

	System.out.print(">\n");
    } 

    // characters inside an element
    public void characters(char[] ch, int start, int length) {
	String chData = (new String(ch, start, length)).trim();
	System.out.println(chData);
    }

    // end of element
    public void endElement(String namespaceURI, String localName, String qName) { 
	System.out.println( "</" + qName + ">\n"); 
    } 

    // end of document
    public void endDocument() {
	System.out.println("--- End parsing document ---");
    }

    // handle fatal errors
    public void fatalError(SAXParseException e) { 
	System.err.println("SimpleJAXPSAXParser - ratal error: " + e); 
    }

    // handle errors
    public void error(SAXParseException e) { 
	System.err.println("SimpleJAXPSAXParser - error: " + e); 
    }
    
    // handle warnings
    public void warning(SAXParseException e) { 
	System.err.println("SimpleJAXPSAXParser - warning: " + e); 
    }

}








