/**************************************************
VU XML, XSL and Web Applications

Lab 3

Name: Wolfgang Aigner
Matr.Nr.: 9755342

Created at: 7.11.2001
Last change: 7.11.2001
**************************************************/

package DOMExample1;

import org.apache.xml.serialize.*;
import org.xml.sax.SAXException;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import java.io.*;

public class DOMExample1 implements IDOMExample1 {

    private DocumentBuilderFactory dbf = null;
    private DocumentBuilder db = null;

    // Constructor
    public DOMExample1(boolean validation) throws ParserConfigurationException{
	// create a DocumentBuilderFactory & configure it
	dbf = DocumentBuilderFactory.newInstance();
	dbf.setValidating(validation);

	// create a DocumentBuilder that satisfies teh
	// constraints specified by the dbf
	db = dbf.newDocumentBuilder();
    }

    // Main method - Creates a parser and parses the xml file given by the command line
    public static void main (String[] args) {
	try {
	    DOMExample1 sp = new DOMExample1(false);
	    sp.printElements(sp.parse(args[0]), args[1]);
        }catch(Exception e) {
	    System.err.println("SimpleJAXPDOMParser - program error: " + e);
	}
    }

    // parse and return the document specified in <filename>
    // if an error occurs, return null;
    public Document parse(String filename) {
	try {
	    return db.parse(new File(filename));
	}catch(SAXException se) {
	    System.err.println("SimpleJAXPDOMParser - parse error: " + se);
	    return null;
	}catch(IOException ie) {
	    System.err.println("SimpleJAXPDOMParser - IO error: " + ie);
	    return null;
	}
    }

    // print all elements which contain an attribute with value attrValue
    public void printElements(Document doc, String attrValue) {
	if(doc!=null) {
	    // get root element
	    Node docElement = doc.getDocumentElement();
	    // print nodes recursively
	    this.printElement((Element)docElement, attrValue);
	}
    }

    // print actual node and all child nodes which contain an attribute with value attrValue
    public void printElement(Element n, String attrValue) {
	boolean printIt = false;
	
	if(attrValue==null || attrValue.equals("")) {
	    printIt = true;
	}
	else {
	    // check attributes
	    if(n.hasAttributes()) {
		NamedNodeMap nnm = n.getAttributes();
		if(nnm!=null) {
		    for(int i=0; i<nnm.getLength(); i++) {
			if(nnm.item(i).getNodeValue().equals(attrValue)) printIt = true;
		    }
		}
	    }
	}
	
	// print opening tag
	if(printIt) System.out.println("<" + n.getNodeName() + ">");

	// get the children of the element
	NodeList nl = n.getChildNodes();

	// print all child names of element children
	Node child = null;
	for(int i=0; i<nl.getLength(); ++i) {
	    child = nl.item(i);
	    if(child.getNodeType() == Node.ELEMENT_NODE) {
		this.printElement((Element)child, attrValue);
	    }
	}

	// print closing tag
	if(printIt) System.out.println("</" + n.getNodeName() + ">");
	
    }
}









