/**************************************************
VU XML, XSL and Web Applications

Lab 2

Name: Wolfgang Aigner
Matr.Nr.: 9755342

Created at: 31.10.2001
Last change: 31.10.2001
**************************************************/

package SimpleParser;

import org.apache.xml.serialize.*;
import org.xml.sax.SAXException;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import java.io.*;

public class SimpleJAXPDOMParser implements ISimpleDOMParser {

    private DocumentBuilderFactory dbf = null;
    private DocumentBuilder db = null;

    // Constructor
    public SimpleJAXPDOMParser(boolean validation) throws ParserConfigurationException{
	// create a DocumentBuilderFactory & configure it
	dbf = DocumentBuilderFactory.newInstance();
	dbf.setValidating(validation);

	// create a DocumentBuilder that satisfies teh
	// constraints specified by the dbf
	db = dbf.newDocumentBuilder();
    }

    // Main method - Creates a parser and parses the xml file given by the command line
    public static void main (String[] args) {
	try {
	    SimpleJAXPDOMParser sp = new SimpleJAXPDOMParser(true);
	    sp.printDocument(sp.parse(args[0]));
        }catch(Exception e) {
	    System.err.println("SimpleJAXPDOMParser - program error: " + e);
	}
    }

    // parse and return the document specified in <filename>
    // if an error occurs, return null;
    public Document parse(String filename) {
	try {
	    return db.parse(new File(filename));
	}catch(SAXException se) {
	    System.err.println("SimpleJAXPDOMParser - parse error: " + se);
	    return null;
	}catch(IOException ie) {
	    System.err.println("SimpleJAXPDOMParser - IO error: " + ie);
	    return null;
	}
    }

    // print the document specified in <filename> to the console
    // check if doc is not null
    public void printDocument(Document doc) {
	if(doc!=null) {
	    // get root element
	    Node docElement = doc.getDocumentElement();
	    // print nodes recursively
	    this.printElement((Element)docElement);
	}
    }

    // print actual node and all child nodes
    public void printElement(Element n) {
	// print opening tag
	System.out.print("<" + n.getNodeName());
	// print attributes
	if(n.hasAttributes()) {
	    NamedNodeMap nnm = n.getAttributes();
	    if(nnm!=null) {
		for(int i=0; i<nnm.getLength(); i++) {
		    Node attr = nnm.item(i);
		    System.out.print(" " + attr.getNodeName()+ "=\"" + attr.getNodeValue() + "\"");
		}
	    }
	}
	
	// get the children of the element
	NodeList nl = n.getChildNodes();

	if(nl==null || nl.getLength()<=0) {
	    System.out.print(" />");
	}
	else {
	    System.out.print(">");

	    // print all child names of element children
	    Node child = null;
	    for(int i=0; i<nl.getLength(); ++i) {
		child = nl.item(i);
		if(child.getNodeType() == Node.ELEMENT_NODE) {
		    this.printElement((Element)child);
		}
		else if(child.getNodeType() == Node.CDATA_SECTION_NODE ||
			child.getNodeType() == Node.TEXT_NODE) {
		    System.out.print(child.getNodeValue());
		}
	    }

	    // print closing tag
	    System.out.print("</" + n.getNodeName() + ">");
	}
    }
}









