/**
*  Class representing a Milestone
*  
*  @author Wolfgang Aigner
*  @version 0.1
*  
*  created: 2004-06-06
*  last change: 2004-06-06
*/

package at.ac.tuwien.cs.ztimeview.core;

import java.util.Date;
import javax.swing.tree.DefaultMutableTreeNode;

import at.ac.tuwien.cs.ztimeview.time.Event;

public class Milestone extends DefaultMutableTreeNode implements Event, TimeElement {
	
	String name = null; // task name
	Date time = null; //event time
			
	//////////////////// CONSTRUCTORS ////////////////////
	
	/** Default constructor */
	public Milestone() throws IllegalArgumentException {
		this("", null);
	}
	
	/** Constructor
	@param name milestone name
	@param time time of event */
	public Milestone(String name, Date time) throws IllegalArgumentException {
		super(null, false);
		this.setName(name);
		this.setTime(time);
	}
	
	//////////////////// METHODS ////////////////////
	
	/** Object to String
	@return  String information of object */
	public String toString() {
		return this.getName() + " (" + this.getTime() + ")";
	}
	
	/** Print task hierarchy textually to the screen
	@param indent number of spaces for indentation */
	public void printHierarchy(int indent) {
		// --- make indent ---
		for(int i=0; i<indent; i++) System.out.print(" ");
		
		// --- print task name, start and end ---
		System.out.print(this.toString() + "\n");
	}
	
	//////////////////// GET/SET METHODS ////////////////////
	
	/** Set task name
	@param name task name */
	public void setName(String name) {
		this.name = name;
	}
	
	/** Get task name
	@return task name */
	public String getName() {
		return name;
	}
	
	//////////////////// Event METHODS ////////////////////
	
	/** Set the time of the event 
	@param time of event */
	public void setTime(Date time) {
		this.time = time;
	}
	
	/** Get the time of the event 
	@return time of event */
	public Date getTime() {
		return this.time;
	}

}
