/**
*  defines axis and calculates interval
*  for ticks according to current start and
*  end value
*  
*  @author Martin Tomitsch
*  @version 0.1
*  
*  created: 2004-06-05
*  last change: 2004-06-06
*/

package at.ac.tuwien.cs.ztimeview.ui;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Vector;
import java.lang.String;
import java.lang.Long;
import java.lang.Math;

public class PAxis {

	private Date start, end;
	private double axisLength, axisX;
	private int LABELSIZE = 50; //length of label text
	
	public PAxis(long start, long end, double axisLength, double axisX) {
		this.start = new Date(start); 
		this.end = new Date(end); 
		this.axisLength = axisLength;
		this.axisX = axisX;
	}
	
	// methods
	
	/** calculates the tick intervals and returns them as a list
	@param */
	Vector calculateTicks() {
		Vector ticks = new Vector();
		PTick tick = new PTick();
		
		//first value
		tick.setLabel(dateToLabel(start.getTime()));
		tick.setCanvasX(axisX);
		ticks.add(tick);
		
		//save ticks for days
		long dateInterval = (long)((end.getTime() - start.getTime()) / 86400000); //interval between dates in days
		int maxNumber = (int)((axisLength + LABELSIZE) / (1.5 * LABELSIZE)); //maximal possible number of labels that can be displayed
		
		//determain largest possible number of labels that can be displayed
		Vector factors = getPrimeFactors(dateInterval);
		int number = 1;
		while (!factors.isEmpty()) {
			if (((Long)(factors.lastElement())).intValue() <= maxNumber) {
				number = ((Long)(factors.lastElement())).intValue();
				factors.removeAllElements();
			}
			else
				factors.remove(factors.lastElement());
		}
		
		double scaleInterval = axisLength / dateInterval;
		
		for(int i = 1; i <= dateInterval; i++) {
			tick = new PTick();
			tick.setCanvasX(axisX+i*scaleInterval);
			if ((i % (int)(dateInterval/number)) == 0) {
				//add label
				tick.setLabel(dateToLabel((long)(((((double)i/(double)dateInterval) * (end.getTime() - start.getTime())) + start.getTime()))));
			}
			else
				tick.setLabel("");
			ticks.add(tick);			
		}
		
		//last value
		tick.setLabel(dateToLabel(end.getTime()));
		tick.setCanvasX(axisX+axisLength);
		ticks.add(tick);
				
		return ticks;
	}
	
	/** converts an element of type Date to String
	@param date
	@return String representing the given date */
	public String dateToLabel(long date) {
		GregorianCalendar c = new GregorianCalendar();
		c.setTime(new Date(date));
		return Integer.toString(c.get(c.DAY_OF_MONTH))
			+ '.' 
			+ Integer.toString(c.get(c.MONTH) + 1) // return value is pointer into field, starting with 0
			+ '.' 
			+ Integer.toString(c.get(c.YEAR));
	}
	
	/** determines all factors of a number
	adapted from http://www.merriampark.com/factor.htm
	@param n interval in days */
	public Vector getPrimeFactors (long n) {
		Vector trialDivisors = null;
		long nDivisions = 0;
		Vector factors = new Vector ();
		nDivisions = 0;
		// Step 0
		long max = 1000; //works for up to 1000 days
		int sqrtn = (int) Math.ceil (Math.sqrt (max));
		trialDivisors = getPrimes (sqrtn);
		trialDivisors.addElement (new Long (sqrtn));
		// Step 1
		int k = 0;
		long divisor;
		Long divisorObj;
		long quotient;
		long remainder;
		// Step 2
		while (n > 1) {
			// Step 3
			divisorObj = (Long) trialDivisors.elementAt (k);
			divisor = divisorObj.longValue ();
			quotient = (long) Math.floor (n / divisor);
			remainder = n % divisor;
			nDivisions++;			
			// Step 4
			if (remainder == 0) {
				// Step 5
				factors.addElement (new Long (divisor));
				n = quotient;
			}		
			else {
				// Step 6
				if (quotient > divisor) {
					k++;
				}
				else {					
					// Step 7
					factors.addElement (new Long (n));
					break;
				}
			}
		}		
		return factors;		
	}
	
	/** find prime numbers
	adapted from http://www.merriampark.com/factor.htm
	@param upTo upper limit */
	public static Vector getPrimes (int upTo) {
		int size = upTo + 1;
		boolean [] flags = new boolean[size];
		Vector primes = new Vector ();
		double limit = Math.sqrt (size);
		// Set flags
		for (int i = 2; i < size; i++) {
			flags[i] = true;
		}
		// Cross out multiples of 2
		int j = 2;
		for (int i = j + j; i < size; i = i + j) {
			flags[i] = false;
		}
		// Cross out multiples of odd numbers
		for (j = 3; j <= limit; j = j + 2) {
			if (flags[j]) {
				for (int i = j + j; i < size; i = i + j) {
					flags[i] = false;
				}
			}
		}
		// Build list of primes from what is left
		for (int i = 2; i < size; i++) {
			if (flags[i]) {
				primes.addElement (new Long (i));
			}
		}
		return primes;
	}

	//////////////////// GET/SET METHODS ////////////////////
	/** sets the range for the axis
	@param start lower date 
	@param end upper date */
	void setRange(long start, long end) {
		this.start = new Date(start); 
		this.end = new Date(end); 
	}
	
	/** sets length of axis
	@param axisLength */
	void setLength(double axisLength) {
		this.axisLength = axisLength;
	}
	
}