/**
*  Piccolo class for an Event 
*  
*  @author Wolfgang Aigner
*  @version 0.1
*  
*  created: 2004-06-06
*  last change: 2004-06-06
*/
package at.ac.tuwien.cs.ztimeview.ui;

import java.util.Date;
import java.util.Enumeration;
import java.util.ArrayList;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.event.InputEvent;
import javax.swing.tree.MutableTreeNode;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.GeneralPath;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.util.PPaintContext;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.event.PInputEventFilter;

import at.ac.tuwien.cs.ztimeview.time.Interval;
import at.ac.tuwien.cs.ztimeview.time.Event;
import at.ac.tuwien.cs.ztimeview.time.TimeInterval;
import at.ac.tuwien.cs.ztimeview.util.PTimeElementFactory;

public class PEvent extends PNode implements PTimeElement, Event {
	
	// user object - contains user data
	MutableTreeNode userObject = null;
	
	// parent of this lifeline
	PLifeLinePlus parentLine = null;
	
	// title of the line
	String title = null;
	
	// time of event
	Date time = null;
	
	/** Default constructor */
	public PEvent() {
		this("Event", new Date(), null);
	}
	
	/** Constructor
	@param aTitle title of the PEvent
	@param time temporal location
	@param userObject user object for this view */
	public PEvent(String title, Date time, MutableTreeNode userObject) {
		super();
		this.userObject = userObject;
		this.title = title;
		this.time = time;
		
		this.initialize();
	}
		
	/** Initialize UI and child elements */
	private void initialize() {
		// --- add tooltip ---
		this.addClientProperty("tooltip", this.title);
	}
	
	/** Paint Event
	Paints background if line is collapsed. If the line is expanded, only the
	children will be paint.
	@param aPaintContext paint context for drawing */
	public void paint(PPaintContext aPaintContext) {
		super.paint(aPaintContext);
	}

	
	//////////////////// GET/SET METHODS ////////////////////

	/** Get parent LifeLine+ 
	@return parent line */
	public PLifeLinePlus getParentLine() {
		return parentLine;
	}

	/** Set parent LifeLine+
	@return parent line */
	public void setParentLine(PLifeLinePlus parent) {
		parentLine = parent;	
	}

	/** Get title
	@return title of lifeline */
	public String getTitle() {
		return this.title;	
	}
	
	/** Get the user object
	@return user object */
	public MutableTreeNode getUserObject() {
		return this.userObject;
	}
	
	/** Set the default height 
	@param height default height for line */
	public void setDefaultHeight(double height) {	}

	/** Get the default height
	@return default line height */
	public double getDefaultHeight() {
		return 0;
	}
	
	//////////////////// Event METHODS ////////////////////
	
	/** Set the time of the event 
	@param time of event */
	public void setTime(Date time) {
		this.time = time;
	}
	
	/** Get the time of the event 
	@return time of event */
	public Date getTime() {
		return this.time;
	}

	
}
