/**
*  Piccolo class for a LifeLine
*  
*  @author Wolfgang Aigner
*  @version 0.2
*  
*  created: 2004-05-20
*  last change: 2004-06-04
*/
package at.ac.tuwien.cs.ztimeview.ui;

import java.util.Date;
import java.awt.Font;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.Graphics2D;
import java.awt.AlphaComposite;
import java.awt.Composite;
import java.awt.Color;
import java.awt.geom.Point2D;
import java.math.BigDecimal;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PPaintContext;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolox.handles.PBoundsHandle;

import at.ac.tuwien.cs.ztimeview.time.Interval;
import at.ac.tuwien.cs.ztimeview.time.TimeInterval;
import at.ac.tuwien.cs.ztimeview.util.ColorManager;

public class PLifeLine extends PTimeLayer implements PropertyChangeListener {

	/* minimal width for semantic zoom 
		-> lines that are shorter are not painted */
	static int MIN_VISIBLE_WIDTH = 10;
	/* minimal height for semantic zoom */
	static int MIN_VISIBLE_HEIGHT = 10;
	/* transparency factor */
	static float TRANSPARENCY = 0.9f;
	/* additional transparency if children are visible */
	static float TRANSPARENCY_ADD = 0.8f;
	
	PText label = null;
	
	/** Default constructor */
	public PLifeLine() {
		this("LifeLine", new TimeInterval(new Date(), new Date()));
	}
	
	/** Constructor
	@aLabel label for the line
	@aInterval temporal extent */
	public PLifeLine(String aLabel, Interval aInterval) {
		super(aInterval);
		this.addPropertyChangeListener(this);
		
		// --- add tooltip ---
		this.addClientProperty("tooltip", aLabel);
		
		// --- add label ---
		this.label = new PText(aLabel);
		//this.label.setPickable(this.label.getWidth() > this.getWidth());
		this.label.setPickable(false);
		this.addChild(this.label);
	}
	
	/** Set font size according to height of line
	@pce event object */
	public void propertyChange(PropertyChangeEvent pce) {
		boolean isRelevantBoundsEvent = (PNode.PROPERTY_BOUNDS == pce.getPropertyName() 
			|| PNode.PROPERTY_FULL_BOUNDS == pce.getPropertyName());		
		if (isRelevantBoundsEvent) {
			this.label.setFont(this.label.getFont().deriveFont((float)(this.getHeight()*0.7)));
		} 		
	}	

	/** Paint me (incl. semantic zoom)
	@aPaintContext paint context to draw on */
	public void paint(PPaintContext aPaintContext) {
		double s = aPaintContext.getScale();
		Graphics2D g2 = aPaintContext.getGraphics();
				
		if(this.paintMe(s)) {	
			this.label.setVisible((this.getHeight()*s) > MIN_VISIBLE_HEIGHT);	
			this.setChildrenPickable(true);
			
			aPaintContext.pushTransparency(TRANSPARENCY);
			if(paintChildren(s)) { // add more transparency if children are visible
				aPaintContext.pushTransparency(TRANSPARENCY_ADD);
				this.label.setTransparency(TRANSPARENCY_ADD);
			}	
			// --- do painting ---
			if (this.getPaint() != null) {
				g2.setPaint(this.getPaint());
				PBounds paintBounds = this.getBounds();
				paintBounds.setSize(paintBounds.getWidth(), this.getFullBoundsReference().getHeight());
				g2.fill(paintBounds);
			}
			//super.paint(aPaintContext);
			
			if(paintChildren(s)) {
				aPaintContext.popTransparency(TRANSPARENCY_ADD);	
			}
			aPaintContext.popTransparency(TRANSPARENCY);
		} else {
			this.setChildrenPickable(false);	
			this.label.setVisible(false);
		}
	}
	
	/** Determines if this node should be painted
	@param scale current scale factor
	@return node to be painted */
	public boolean paintMe(double scale) {
		return (((this.getWidth()*scale) > MIN_VISIBLE_WIDTH) ||
			((this.getHeight()*scale) > MIN_VISIBLE_HEIGHT));
	}
	
	/** Check, if a child is painted
	@param scale current scale factor
	@return if a child is going to be painted */
	public boolean paintChildren(double scale) {
		int count = getChildrenCount();
		int i = 0;
		boolean paintChild = false;
		
		while(i<count && !paintChild) {
			PNode each = (PNode) getChild(i);
			
			if(each instanceof PLifeLine) {
					paintChild = ((PLifeLine) each).paintMe(scale);
			}
			
			i++;
		}	
		
		return paintChild;
	}
	
}
