/**
*  Piccolo Layer containing the scale
*  
*  @author Martin Tomitsch
*  @version 0.2
*  
*  created: 2004-06-03
*  last change: 2004-06-05
*/
package at.ac.tuwien.cs.ztimeview.ui;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Vector;
import java.util.Iterator;
import java.text.SimpleDateFormat;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.util.PBounds;
import edu.umd.cs.piccolo.util.PPaintContext;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.nodes.PPath;

public class PScaleLayer extends PLayer {
	
	private int BACKGROUNDHEIGHT=50;
	private int LABELSIZE = 50; //length of label text
	private int AXISTOP = 20; //top y position of axis
	Date scaleStart = new Date();
	Date scaleEnd = new Date();
	double canvasWidth;
		
	// precision of display width[px] = duration[ms]*PRECISION
	private double PRECISION = 0.000001; 
	
	public PScaleLayer(double canvasWidth, Date start, Date end) {
		super();
		scaleStart = start;
		scaleEnd = end;
		this.canvasWidth = canvasWidth;
	}
	
	/** paint axis: creating children, calculating tick intervals, drawing axis and labels
	@param start lower date
	@param end upper date
	@param bounds */
	public void drawAxis (long start, long end, long length, long x, double space) {
		PPath background;
		background = PPath.createRectangle(0, 0, (float)canvasWidth, (float)BACKGROUNDHEIGHT);
		background.setPaint(Color.WHITE);
		background.setStroke(null);
		this.addChild(0,background);
		
		PPath axisLine = new PPath();
		//axisLine.setStrokePaint(Color.darkGray);
		axisLine.moveTo((float)(canvasWidth*space),45);
		axisLine.lineTo((float)(canvasWidth*space),AXISTOP);
		axisLine.lineTo((float)(canvasWidth-canvasWidth*space),AXISTOP);
		axisLine.lineTo((float)(canvasWidth-canvasWidth*space),45);
		
		PAxis axis = new PAxis(start, end, canvasWidth-2*canvasWidth*space, canvasWidth*space);
		Vector ticks = new Vector(axis.calculateTicks()); //list of ticks
		PTick tick = new PTick();
		//Vector labels = new Vector(); //list of PText elements 
		PText label;
		
		Iterator each = ticks.iterator();
		while (each.hasNext()) {
			tick = (PTick)each.next();
			//write label text
			if (tick.getLabel().compareTo("") != 0) {
				label = new PText(tick.getLabel());
				//label.setStrokePaint(Color.darkGray);
				label.setBounds(tick.getCanvasX()-LABELSIZE/2,3,label.getBounds().getSize().getWidth(),label.getBounds().getSize().getHeight());
				this.addChild(label);
				//draw additional tick for the label (above the axis line)
				axisLine.moveTo((float)tick.getCanvasX(),AXISTOP);
				axisLine.lineTo((float)tick.getCanvasX(),AXISTOP-4);
			}
			//draw tick
			axisLine.moveTo((float)tick.getCanvasX(),AXISTOP);
			axisLine.lineTo((float)tick.getCanvasX(),AXISTOP+15);
		}

		this.addChild(axisLine);
				
		//PText label = new PText(dateToLabel(start));
		//labels.add(label);
		//this.addChild(2,label);
	}
	
	/** converts an element of type Date to String
	@param date
	@return String representing the given date */
	public String dateToLabel(long date) {
		GregorianCalendar c = new GregorianCalendar();
		c.setTime(new Date(date));
		return Integer.toString(c.get(c.DAY_OF_MONTH))
			+ '.' 
			+ Integer.toString(c.get(c.MONTH) + 1) // return value is pointer into field, starting with 0
			+ '.' 
			+ Integer.toString(c.get(c.YEAR));
	}
	
	//****************************************************************
	// Structure - Methods for manipulating and traversing the 
	// parent child relationship
	// 
	// Most of these methods won't need to be overridden by subclasses
	// but you will use them frequently to build up your node structures.
	//****************************************************************
	 
	/**
	 * Add a node to be a new child of this node at the specified index.
	 * If child was previously a child of another node, it is removed 
	 * from that node first.
	 * 
	 * @param child the new child to add to this node
	 */
	public void addChild(int index, PNode child) {
		super.addChild(index, child);
	}
	
	public void fullPaint(PPaintContext aPaintContext) {
		double s = aPaintContext.getScale();
		PBounds vb = aPaintContext.getCamera().getViewBounds();
		
		long x = (long) (vb.getOrigin().getX() / PRECISION); 
		long width = (long) (vb.getSize().getWidth() / PRECISION);
		double space = 0.05; // space to the left and right of the axis (in percent)
		
		Date scaleCurrentStart = new Date(scaleStart.getTime() + x + (long)(width*space));
		Date scaleCurrentEnd = new Date(scaleStart.getTime() + x + width - (long)(width*space));
		
		// delete all children first
		this.removeAllChildren();
		
		drawAxis(scaleCurrentStart.getTime(), scaleCurrentEnd.getTime(), width, x, space);
		
		//GregorianCalendar c = new GregorianCalendar();

		/*c.setTime(scaleCurrentEnd);
		label2.setBounds(aPaintContext.getLocalClip().getWidth() - label2.getWidth(), this.getY(), this.getWidth(), this.getHeight());	
		label2.setText(Integer.toString(c.get(c.DAY_OF_MONTH))
			+ '.' 
			+ Integer.toString(c.get(c.MONTH) + 1) // return value is pointer into field, starting with 0
			+ '.' 
			+ Integer.toString(c.get(c.YEAR)));*/
		
		super.fullPaint(aPaintContext);
	}
	
	
	//////////////////// GET/SET METHODS ////////////////////
}
